@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: pkcs11_lib.c 379 2009-12-03 08:40:22Z ludovic.rousseau $
+ * $Id: pkcs11_lib.c 435 2010-06-07 14:53:17Z ludovic.rousseau $
  */
 
 #define __PKCS11_LIB_C__
@@ -237,7 +237,7 @@ static SECMODModule *find_module_by_library(char *pkcs11_module)
  * to load, load it, otherwize select on of the standard modules from the
  * secmod.db list.
  */
-int load_pkcs11_module(char *pkcs11_module, pkcs11_handle_t **hp)
+int load_pkcs11_module(const char *pkcs11_module, pkcs11_handle_t **hp)
 {
   pkcs11_handle_t *h = (pkcs11_handle_t *)calloc(sizeof(pkcs11_handle_t),1);
   SECMODModule *module = NULL;
@@ -261,7 +261,7 @@ int load_pkcs11_module(char *pkcs11_module, pkcs11_handle_t **hp)
   }
 
   /* specified module is not already loaded, load it now */
-  moduleSpec = (char *)malloc(sizeof(SPEC_TEMPLATE) + strlen(pkcs11_module));
+  moduleSpec = malloc(sizeof(SPEC_TEMPLATE) + strlen(pkcs11_module));
   if (!moduleSpec) {
     DBG1("Malloc failed when allocating module spec", strerror(errno));
     free (h);
@@ -662,6 +662,26 @@ int pkcs11_login(pkcs11_handle_t *h, char *password)
   return (rv == SECSuccess) ? 0 : -1;
 }
 
+int get_slot_login_required(pkcs11_handle_t *h)
+{
+  if (h->slot == NULL) {
+    DBG("Login failed: No Slot selected");
+    return -1;
+  }
+
+  return PK11_NeedLogin(h->slot);
+}
+
+int get_slot_protected_authentication_path(pkcs11_handle_t *h)
+{
+  if (h->slot == NULL) {
+    DBG("Login failed: No Slot selected");
+    return -1;
+  }
+
+  return PK11_ProtectedAuthenticationPath(h->slot);
+}
+
 int close_pkcs11_session(pkcs11_handle_t *h)
 {
   if (h->slot) {
@@ -743,7 +763,7 @@ cert_object_t **get_certificate_list(pkcs11_handle_t *h, int *count)
     return NULL;
   }
 
-  certs = (cert_object_t **)malloc(sizeof(cert_object_t *)*certCount);
+  certs = malloc(sizeof(cert_object_t *)*certCount);
   if (certs == NULL) {
     return NULL;
   }
@@ -934,12 +954,13 @@ struct pkcs11_handle_str {
 int crypto_init(cert_policy *policy)
 {
   /* arg is ignored for OPENSSL */
+  (void)policy;
   OpenSSL_add_all_algorithms();
   ERR_load_crypto_strings();
   return 0;
 }
 
-int load_pkcs11_module(char *module, pkcs11_handle_t **hp)
+int load_pkcs11_module(const char *module, pkcs11_handle_t **hp)
 {
   int rv;
   struct stat module_stat;
@@ -1047,18 +1068,19 @@ int init_pkcs11_module(pkcs11_handle_t *h,int flag)
   CK_ULONG i;
   CK_SLOT_ID_PTR slots;
   CK_INFO info;
-  CK_C_INITIALIZE_ARGS initArgs;
   /*
    Set up arguments to allow native threads
    According with pkcs#11v2.20, must set all pointers to null
    and flags CKF_OS_LOCKING_OK
   */
-  initArgs.CreateMutex = NULL;
-  initArgs.DestroyMutex = NULL;
-  initArgs.LockMutex = NULL;
-  initArgs.UnlockMutex = NULL;
-  initArgs.flags = CKF_OS_LOCKING_OK;
-  initArgs.CreateMutex = NULL;
+  CK_C_INITIALIZE_ARGS initArgs = {
+	.CreateMutex = NULL,
+	.DestroyMutex = NULL,
+	.LockMutex = NULL,
+	.UnlockMutex = NULL,
+	.flags = CKF_OS_LOCKING_OK,
+	.pReserved = NULL
+  };
 
   /* initialise the module */
   if (flag) rv = h->fl->C_Initialize((CK_VOID_PTR) &initArgs);
@@ -1260,7 +1282,7 @@ find_slot_by_slotlabel_and_tokenlabel(pkcs11_handle_t *h,
   if (strcmp(wanted_slot_label, "none") == 0) {
     for (i= 0; i < h->slot_count; i++) {
       if (h->slots[i].token_present &&
-	  strcmp(wanted_token_label, h->slots[i].label) == 0) {
+	  strcmp(wanted_token_label, (char *)h->slots[i].label) == 0) {
 	*slot_num = i;
 	return (0);
       }
@@ -1356,7 +1378,10 @@ int pkcs11_login(pkcs11_handle_t *h, char *password)
   int rv;
 
   DBG("login as user CKU_USER");
-  rv = h->fl->C_Login(h->session, CKU_USER, (unsigned char*)password, strlen(password));
+  if (password)
+	  rv = h->fl->C_Login(h->session, CKU_USER, (unsigned char*)password, strlen(password));
+  else
+	  rv = h->fl->C_Login(h->session, CKU_USER, NULL, 0);
   if (rv != CKR_OK) {
     set_error("C_Login() failed: 0x%08lX", rv);
     return -1;
@@ -1377,6 +1402,19 @@ int get_slot_login_required(pkcs11_handle_t *h)
   return tinfo.flags & CKF_LOGIN_REQUIRED;
 }
 
+int get_slot_protected_authentication_path(pkcs11_handle_t *h)
+{
+  int rv;
+  CK_TOKEN_INFO tinfo;
+
+  rv = h->fl->C_GetTokenInfo(h->slots[h->current_slot].id, &tinfo);
+  if (rv != CKR_OK) {
+    set_error("C_GetTokenInfo() failed: 0x%08lX", rv);
+    return -1;
+  }
+  return tinfo.flags & CKF_PROTECTED_AUTHENTICATION_PATH;
+}
+
 static void free_certs(cert_object_t **certs, int cert_count)
 {
   int i;
@@ -1516,7 +1554,7 @@ cert_object_t **get_certificate_list(pkcs11_handle_t *h, int *ncerts)
     /* Pass 3: store certificate */
 
     /* convert to X509 data structure */
-      x509 = d2i_X509(NULL, (CK_BYTE **)&cert_template[3].pValue, cert_template[3].ulValueLen);
+      x509 = d2i_X509(NULL, (const unsigned char **)&cert_template[3].pValue, cert_template[3].ulValueLen);
       if (x509 == NULL) {
         free(id_value);
         free(cert_value);
