@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: uri.c 264 2007-04-12 14:14:49Z ludovic.rousseau $
+ * $Id: uri.c 424 2010-04-10 14:52:33Z ludovic.rousseau $
  */
 
 #define __URI_C_
@@ -26,10 +26,10 @@
 #include "error.h"
 #include "strings.h"
 
-static const char *valid_urls[]= 
+static const char *valid_urls[]=
 		{"file:///","http://","https://","ftp://","ldap://",NULL};
-/* 
-comodity functions 
+/*
+comodity functions
 Analize provided pathname and check type
 Returns 1 on true, 0 on false, -1 on error
 */
@@ -339,7 +339,7 @@ static int get_file(uri_t *uri, unsigned char **data, ssize_t * length)
     set_error("lseek() failed: %s", strerror(errno));
     return -1;
   }
-  *data = (unsigned char *)malloc(*length);
+  *data = malloc(*length);
   if (*data == NULL) {
     close(fd);
     set_error("not enough free memory available");
@@ -414,7 +414,7 @@ static int get_http(uri_t *uri, unsigned char **data, size_t *length, int rec_le
   /* receive response */
   DBG("receiving...");
   bufsize = 128;
-  buf = (unsigned char *)malloc(bufsize);
+  buf = malloc(bufsize);
   if (buf == NULL) {
     close(sock);
     set_error("not enough free memory available");
@@ -504,7 +504,7 @@ static int get_http(uri_t *uri, unsigned char **data, size_t *length, int rec_le
     set_error("no data received");
     return -1;
   }
-  *data = (unsigned char *)malloc(*length);
+  *data = malloc(*length);
   if (*data == NULL) {
     free(buf);
     set_error("not enough free memory available");
@@ -562,7 +562,7 @@ static int get_ldap(uri_t *uri, unsigned char **data, size_t *length)
   /* allocate memory and copy the item */
   DBG("copying data...");
   *length = (*vals)->bv_len;
-  *data = (unsigned char *)malloc(*length);
+  *data = malloc(*length);
   if (*data == NULL) {
     ldap_value_free_len(vals);
     ldap_msgfree(msg);
@@ -613,6 +613,7 @@ int get_from_uri(const char *str, unsigned char **data, size_t *length)
       set_error("Compiled without LDAP support");
 #endif
       break;
+	case unknown:
     default:
       set_error("unsupported protocol");
       rv = -1;
