@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: pam_config.h 270 2007-05-21 08:13:00Z ludovic.rousseau $
+ * $Id: pam_config.h 414 2010-04-07 14:09:42Z ludovic.rousseau $
  */
 
 /*
@@ -26,7 +26,7 @@
 #include "../common/cert_vfy.h"
 
 struct configuration_st {
-	char * config_file;
+	const char *config_file;
 	scconf_context *ctx;
 	int debug;
 	int nullok;
@@ -35,13 +35,16 @@ struct configuration_st {
 	int use_authok;
 	int card_only;
 	int wait_for_card;
-	char *pkcs11_module;
-	char *pkcs11_modulepath;
-	char **screen_savers;
+	const char *pkcs11_module;
+	const char *pkcs11_modulepath;
+	const char **screen_savers;
+	const char *slot_description;
 	int slot_num;
 	int support_threads;
 	cert_policy policy;
-	char *username; /* provided user name */
+	const char *token_type;
+	const char *username; /* provided user name */
+	int quiet;
 };
 
 struct configuration_st *pk_configure( int argc, const char **argv );
