@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: pam_pkcs11.c 281 2007-05-22 08:28:40Z ludovic.rousseau $
+ * $Id: pam_pkcs11.c 437 2010-06-08 08:01:32Z ludovic.rousseau $
  */
 
 /* We have to make this definitions before we include the pam header files! */
@@ -43,7 +43,7 @@
 #include "pam_config.h"
 #include "mapper_mgr.h"
 
-#if ENABLE_NLS
+#ifdef ENABLE_NLS
 #include <libintl.h>
 #include <locale.h>
 #define _(string) gettext(string)
@@ -59,7 +59,7 @@
 /*
 * comodity function that returns 1 on null, empty o spaced string
 */
-int is_spaced_str(const char *str) {
+static int is_spaced_str(const char *str) {
 	char *pt=(char *)str;
 	if(!str) return 1;
 	if (!strcmp(str,"")) return 1;
@@ -83,12 +83,12 @@ static int pam_prompt(pam_handle_t *pamh, int style, char **response, char *text
 
   msg.msg_style = style;
   msg.msg = text;
-  rv = pam_get_item(pamh, PAM_CONV, (const void **) &conv);
+  rv = pam_get_item(pamh, PAM_CONV, &conv);
   if (rv != PAM_SUCCESS)
     return rv;
   if ((conv == NULL) || (conv->conv == NULL))
     return PAM_CRED_INSUFFICIENT;
-  rv = conv->conv(1, (const struct pam_message **)msgp, &resp, conv->appdata_ptr);
+  rv = conv->conv(1, msgp, &resp, conv->appdata_ptr);
   if (rv != PAM_SUCCESS)
     return rv;
   if ((resp == NULL) || (resp[0].resp == NULL))
@@ -102,7 +102,7 @@ static int pam_prompt(pam_handle_t *pamh, int style, char **response, char *text
   return PAM_SUCCESS;
 }
 
-static void 
+static void
 pam_syslog(pam_handle_t *pamh, int priority, const char *fmt, ...)
 {
     va_list ap;
@@ -132,7 +132,7 @@ static int pam_get_pwd(pam_handle_t *pamh, char **pwd, char *text, int oitem, in
   /* use stored password if variable oitem is set */
   if ((oitem == PAM_AUTHTOK) || (oitem == PAM_OLDAUTHTOK)) {
     /* try to get stored item */
-    rv = pam_get_item(pamh, oitem, (const void **) &old_pwd);
+    rv = pam_get_item(pamh, oitem, &old_pwd);
     if (rv != PAM_SUCCESS)
       return rv;
     if (old_pwd != NULL) {
@@ -145,12 +145,12 @@ static int pam_get_pwd(pam_handle_t *pamh, char **pwd, char *text, int oitem, in
   if (text != NULL) {
     msg.msg_style = PAM_PROMPT_ECHO_OFF;
     msg.msg = text;
-    rv = pam_get_item(pamh, PAM_CONV, (const void **) &conv);
+    rv = pam_get_item(pamh, PAM_CONV, &conv);
     if (rv != PAM_SUCCESS)
       return rv;
     if ((conv == NULL) || (conv->conv == NULL))
       return PAM_CRED_INSUFFICIENT;
-    rv = conv->conv(1, (const struct pam_message **)msgp, &resp, conv->appdata_ptr);
+    rv = conv->conv(1, msgp, &resp, conv->appdata_ptr);
     if (rv != PAM_SUCCESS)
       return rv;
     if ((resp == NULL) || (resp[0].resp == NULL))
@@ -175,7 +175,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
   int i, rv;
   const char *user = NULL;
   char *password;
-  char password_prompt[70];
+  char password_prompt[128];
   unsigned int slot_num = 0;
   int is_a_screen_saver = 0;
   struct configuration_st *configuration;
@@ -206,19 +206,27 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 	return PAM_AUTHINFO_UNAVAIL;
   }
 
+  /* Either slot_description or slot_num, but not both, needs to be used */
+  if ((configuration->slot_description != NULL && configuration->slot_num != -1) || (configuration->slot_description == NULL && configuration->slot_num == -1)) {
+	ERR("Error setting configuration parameters");
+	return PAM_AUTHINFO_UNAVAIL;
+  }
+
   /* fail if we are using a remote server
    * local login: DISPLAY=:0
    * XDMCP login: DISPLAY=host:0 */
   {
 	  char *display = getenv("DISPLAY");
 
-	  if (display && (display[0] != ':') && (display[0] != '\0'))
+	  if (display)
 	  {
-		ERR1("Remote login (from %s) is not (yet) supported", display);
-		pam_syslog(pamh, LOG_ERR, 
-                        "Remote login (from %s) is not (yet) supported",
-			display);
-		return PAM_AUTHINFO_UNAVAIL;
+		  if (strncmp(display, "localhost:", 10) != 0 && (display[0] != ':')
+			  && (display[0] != '\0')) {
+			  ERR1("Remote login (from %s) is not (yet) supported", display);
+			  pam_syslog(pamh, LOG_ERR,
+				  "Remote login (from %s) is not (yet) supported", display);
+			  return PAM_AUTHINFO_UNAVAIL;
+		  }
 	  }
   }
 
@@ -227,12 +235,13 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
   bindtextdomain(PACKAGE, "/usr/share/locale");
   textdomain(PACKAGE);
 #endif
-  
+
   /* init openssl */
   rv = crypto_init(&configuration->policy);
   if (rv != 0) {
     ERR("Failed to initialize crypto");
-    pam_syslog(pamh,LOG_ERR, "Failed to initialize crypto");
+    if (!configuration->quiet)
+      pam_syslog(pamh,LOG_ERR, "Failed to initialize crypto");
     return PAM_AUTHINFO_UNAVAIL;
   }
 
@@ -247,7 +256,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
    *  1) nothing if card_only isn't set
    *  2) if logged in, block in pam conversation until the token used for login
    *     is inserted
-   *  3) if not logged in, block until a token that could be used for logging in 
+   *  3) if not logged in, block until a token that could be used for logging in
    *     is inserted
    * right now, logged in means PKC11_LOGIN_TOKEN_NAME is set,
    * but we could something else later (like set some per-user state in
@@ -257,7 +266,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 	char *service;
 	if (configuration->screen_savers) {
 	    DBG("Is it a screen saver?");
-    	    rv = pam_get_item(pamh, PAM_SERVICE, (const void **) &service);
+    	    rv = pam_get_item(pamh, PAM_SERVICE, &service);
 	    for (i=0; configuration->screen_savers[i]; i++) {
 		if (strcmp(configuration->screen_savers[i], service) == 0) {
 		    is_a_screen_saver = 1;
@@ -267,22 +276,27 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 	}
 
 	pkcs11_pam_fail = PAM_CRED_INSUFFICIENT;
-        
+
 	/* look to see if username is already set */
-    	rv = pam_get_item(pamh, PAM_USER, (const void **) &user);
+    	rv = pam_get_item(pamh, PAM_USER, &user);
 	if (user) {
 	    DBG1("explicit username = [%s]", user);
-	} 
+	}
   } else {
-        pam_prompt(pamh, PAM_TEXT_INFO, NULL,
-                   _("Please insert your smart card or enter your username."));
-	/* get user name */
-	rv = pam_get_user(pamh, &user, NULL);
+	rv = pam_get_item(pamh, PAM_USER, &user);
+	if (rv != PAM_SUCCESS || user == NULL || user[0] == '\0') {
+	  snprintf(password_prompt, sizeof(password_prompt),
+		  _("Please insert your %s or enter your username."),
+		  _(configuration->token_type));
+	  pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
+	  /* get user name */
+	  rv = pam_get_user(pamh, &user, NULL);
 
-	if (rv != PAM_SUCCESS) {
-	  pam_syslog(pamh, LOG_ERR, 
-                     "pam_get_user() failed %s", pam_strerror(pamh, rv));
-	  return PAM_USER_UNKNOWN;
+	  if (rv != PAM_SUCCESS) {
+		pam_syslog(pamh, LOG_ERR,
+			"pam_get_user() failed %s", pam_strerror(pamh, rv));
+		return PAM_USER_UNKNOWN;
+	  }
 	}
 	DBG1("username = [%s]", user);
   }
@@ -298,8 +312,11 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
   DBG("loading pkcs #11 module...");
   rv = load_pkcs11_module(configuration->pkcs11_modulepath, &ph);
   if (rv != 0) {
-    ERR1("load_pkcs11_module() failed: %s", get_error());
-    pam_syslog(pamh, LOG_ERR, "load_pkcs11_module() failed: %s", get_error());
+    ERR2("load_pkcs11_module() failed loading %s: %s",
+		configuration->pkcs11_modulepath, get_error());
+    if (!configuration->quiet)
+	pam_syslog(pamh, LOG_ERR, "load_pkcs11_module() failed loading %s: %s",
+		configuration->pkcs11_modulepath, get_error());
     return PAM_AUTHINFO_UNAVAIL;
   }
 
@@ -309,16 +326,24 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
   if (rv != 0) {
     release_pkcs11_module(ph);
     ERR1("init_pkcs11_module() failed: %s", get_error());
-    pam_syslog(pamh, LOG_ERR, "init_pkcs11_module() failed: %s", get_error());
+    if (!configuration->quiet)
+      pam_syslog(pamh, LOG_ERR, "init_pkcs11_module() failed: %s", get_error());
     return PAM_AUTHINFO_UNAVAIL;
   }
 
   /* open pkcs #11 session */
-  rv = find_slot_by_number_and_label(ph, configuration->slot_num, 
+  if (configuration->slot_description != NULL) {
+    rv = find_slot_by_slotlabel_and_tokenlabel(ph,
+      configuration->slot_description, login_token_name, &slot_num);
+  } else if (configuration->slot_num != -1) {
+    rv = find_slot_by_number_and_label(ph, configuration->slot_num,
                                      login_token_name, &slot_num);
+  }
+
   if (rv != 0) {
     ERR("no suitable token available");
-    pam_syslog(pamh, LOG_ERR, "no suitable token available");
+    if (!configuration->quiet)
+      pam_syslog(pamh, LOG_ERR, "no suitable token available");
 
     if (!configuration->card_only) {
       release_pkcs11_module(ph);
@@ -329,16 +354,23 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
      * or because we used one to log in */
     if (login_token_name || configuration->wait_for_card) {
       if (login_token_name) {
-        sprintf(password_prompt,
-                _("Please insert your smart card called \"%.32s\"."),
-                login_token_name);
+        snprintf(password_prompt, sizeof(password_prompt),
+			_("Please insert your smart card called \"%.32s\"."),
+			login_token_name);
         pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
       } else {
         pam_prompt(pamh, PAM_TEXT_INFO, NULL,
                  _("Please insert your smart card."));
       }
-      rv = wait_for_token(ph, configuration->slot_num,
+
+      if (configuration->slot_description != NULL) {
+	rv = wait_for_token_by_slotlabel(ph, configuration->slot_description,
+          login_token_name, &slot_num);
+      } else if (configuration->slot_num != -1) {
+        rv = wait_for_token(ph, configuration->slot_num,
                           login_token_name, &slot_num);
+      }
+
       if (rv != 0) {
         release_pkcs11_module(ph);
         return pkcs11_pam_fail;
@@ -350,13 +382,21 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     } else {
       /* we haven't prompted for the user yet, get the user and see if
        * the smart card has been inserted in the mean time */
-      pam_prompt(pamh, PAM_TEXT_INFO, NULL, 
-                 _("Please insert your smart card or enter your username."));
+      snprintf(password_prompt, sizeof(password_prompt),
+	  	_("Please insert your %s or enter your username."),
+		_(configuration->token_type));
+      pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
       rv = pam_get_user(pamh, &user, NULL);
 
       /* check one last time for the smart card before bouncing to the next
        * module */
-      rv = find_slot_by_number(ph, configuration->slot_num, &slot_num);
+      if (configuration->slot_description != NULL) {
+	rv = find_slot_by_slotlabel(ph, configuration->slot_description,
+	  &slot_num);
+      } else if (configuration->slot_num != -1) {
+        rv = find_slot_by_number(ph, configuration->slot_num, &slot_num);
+      }
+
       if (rv != 0) {
         /* user gave us a user id and no smart card go to next module */
         release_pkcs11_module(ph);
@@ -364,65 +404,96 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
       }
     }
   } else {
-      pam_prompt(pamh, PAM_TEXT_INFO, NULL, _("Smart card inserted. "));
+      snprintf(password_prompt, sizeof(password_prompt),
+		  _("Found the %s."), _(configuration->token_type));
+      pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
   }
   rv = open_pkcs11_session(ph, slot_num);
   if (rv != 0) {
     release_pkcs11_module(ph);
     ERR1("open_pkcs11_session() failed: %s", get_error());
-    pam_syslog(pamh, LOG_ERR, "open_pkcs11_session() failed: %s", get_error());
+    if (!configuration->quiet)
+      pam_syslog(pamh, LOG_ERR, "open_pkcs11_session() failed: %s", get_error());
     return pkcs11_pam_fail;
   }
 
-  /* get password */
-  sprintf(password_prompt, _("Welcome %.32s!"), get_slot_label(ph));
-  pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
-  if (configuration->use_first_pass) {
-    rv = pam_get_pwd(pamh, &password, NULL, PAM_AUTHTOK, 0);
-  } else if (configuration->try_first_pass) {
-    rv = pam_get_pwd(pamh, &password, _("Smart card password: "), PAM_AUTHTOK,
-      PAM_AUTHTOK);
-  } else {
-    rv = pam_get_pwd(pamh, &password, _("Smart card password: "), 0,
-      PAM_AUTHTOK);
-  }
-  if (rv != PAM_SUCCESS) {
+  rv = get_slot_login_required(ph);
+  if (rv == -1) {
     release_pkcs11_module(ph);
-    pam_syslog(pamh, LOG_ERR, 
-               "pam_get_pwd() failed: %s", pam_strerror(pamh, rv));
+    ERR1("get_slot_login_required() failed: %s", get_error());
+    if (!configuration->quiet)
+      pam_syslog(pamh, LOG_ERR, "get_slot_login_required() failed: %s", get_error());
     return pkcs11_pam_fail;
-  }
+  } else if (rv) {
+    /* get password */
+	snprintf(password_prompt, sizeof(password_prompt),
+		_("Welcome %.32s!"), get_slot_tokenlabel(ph));
+	pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
+
+	/* no CKF_PROTECTED_AUTHENTICATION_PATH */
+	rv = get_slot_protected_authentication_path(ph);
+	if ((-1 == rv) || (0 == rv))
+	{
+		sprintf(password_prompt, _("%s PIN: "), _(configuration->token_type));
+		if (configuration->use_first_pass) {
+			rv = pam_get_pwd(pamh, &password, NULL, PAM_AUTHTOK, 0);
+		} else if (configuration->try_first_pass) {
+			rv = pam_get_pwd(pamh, &password, password_prompt, PAM_AUTHTOK,
+					PAM_AUTHTOK);
+		} else {
+			rv = pam_get_pwd(pamh, &password, password_prompt, 0, PAM_AUTHTOK);
+		}
+		if (rv != PAM_SUCCESS) {
+			release_pkcs11_module(ph);
+			pam_syslog(pamh, LOG_ERR,
+					"pam_get_pwd() failed: %s", pam_strerror(pamh, rv));
+			return pkcs11_pam_fail;
+		}
 #ifndef DEBUG_HIDE_PASSWORD
-  DBG1("password = [%s]", password);
+		DBG1("password = [%s]", password);
 #endif
 
-  /* check password length */
-  if (!configuration->nullok && strlen(password) == 0) {
-    release_pkcs11_module(ph);
-    memset(password, 0, strlen(password));
-    free(password);
-    pam_syslog(pamh, LOG_ERR, 
-         "password length is zero but the 'nullok' argument was not defined.");
-    return PAM_AUTH_ERR;
-  }
+		/* check password length */
+		if (!configuration->nullok && strlen(password) == 0) {
+			release_pkcs11_module(ph);
+			memset(password, 0, strlen(password));
+			free(password);
+			pam_syslog(pamh, LOG_ERR,
+					"password length is zero but the 'nullok' argument was not defined.");
+			return PAM_AUTH_ERR;
+		}
+	}
+	else
+	{
+		pam_prompt(pamh, PAM_TEXT_INFO, NULL,
+			_("Enter your %s PIN on the pinpad"), _(configuration->token_type));
+		/* use pin pad */
+		password = NULL;
+	}
 
-  /* call pkcs#11 login to ensure that the user is the real owner of the card 
-   * we need to do thise before get_certificate_list because some tokens
-   * can not read their certificates until the token is authenticated */
-  rv = pkcs11_login(ph, password);
-  /* erase and free in-memory password data asap */
-  memset(password, 0, strlen(password));
-  free(password); 
-  if (rv != 0) {
-    ERR1("open_pkcs11_login() failed: %s", get_error());
-    pam_syslog(pamh, LOG_ERR, "open_pkcs11_login() failed: %s", get_error());
-    goto auth_failed_nopw;
+    /* call pkcs#11 login to ensure that the user is the real owner of the card
+     * we need to do thise before get_certificate_list because some tokens
+     * can not read their certificates until the token is authenticated */
+    rv = pkcs11_login(ph, password);
+    /* erase and free in-memory password data asap */
+	if (password)
+	{
+		memset(password, 0, strlen(password));
+		free(password);
+	}
+    if (rv != 0) {
+      ERR1("open_pkcs11_login() failed: %s", get_error());
+      if (!configuration->quiet)
+        pam_syslog(pamh, LOG_ERR, "open_pkcs11_login() failed: %s", get_error());
+      goto auth_failed_nopw;
+    }
   }
 
   cert_list = get_certificate_list(ph, &ncert);
   if (rv<0) {
     ERR1("get_certificate_list() failed: %s", get_error());
-    pam_syslog(pamh, LOG_ERR, "get_certificate_list() failed: %s", get_error());
+    if (!configuration->quiet)
+      pam_syslog(pamh, LOG_ERR, "get_certificate_list() failed: %s", get_error());
     goto auth_failed_nopw;
   }
 
@@ -439,7 +510,8 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
       rv = verify_certificate(x509,&configuration->policy);
       if (rv < 0) {
         ERR1("verify_certificate() failed: %s", get_error());
-        pam_syslog(pamh, LOG_ERR, 
+        if (!configuration->quiet)
+          pam_syslog(pamh, LOG_ERR,
                    "verify_certificate() failed: %s", get_error());
 	goto auth_failed_nopw;
       } else if (rv != 1) {
@@ -450,7 +522,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     /* CA and CRL verified, now check/find user */
 
     if ( is_spaced_str(user) ) {
-      /* 
+      /*
 	if provided user is null or empty extract and set user
 	name from certificate
       */
@@ -458,7 +530,8 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 	user=find_user(x509);
 	if (!user) {
           ERR2("find_user() failed: %s on cert #%d", get_error(),i+1);
-          pam_syslog(pamh, LOG_ERR,
+          if (!configuration->quiet)
+            pam_syslog(pamh, LOG_ERR,
                      "find_user() failed: %s on cert #%d",get_error(),i+1);
 	  continue; /* try on next certificate */
 	} else {
@@ -467,7 +540,8 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
   	  rv = pam_set_item(pamh, PAM_USER,(const void *)user);
 	  if (rv != PAM_SUCCESS) {
 	    ERR1("pam_set_item() failed %s", pam_strerror(pamh, rv));
-	    pam_syslog(pamh, LOG_ERR, 
+            if (!configuration->quiet)
+	      pam_syslog(pamh, LOG_ERR,
                        "pam_set_item() failed %s", pam_strerror(pamh, rv));
 	    goto auth_failed_nopw;
 	}
@@ -480,7 +554,8 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
         rv = match_user(x509, user);
         if (rv < 0) { /* match error; abort and return */
           ERR1("match_user() failed: %s", get_error());
-          pam_syslog(pamh, LOG_ERR, "match_user() failed: %s", get_error());
+          if (!configuration->quiet)
+            pam_syslog(pamh, LOG_ERR, "match_user() failed: %s", get_error());
 	  goto auth_failed_nopw;
         } else if (rv == 0) { /* match didn't success */
           DBG("certificate is valid but does not match the user");
@@ -496,7 +571,8 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
   /* now myCert points to our found certificate or null if no user found */
   if (!chosen_cert) {
     ERR("no valid certificate which meets all requirements found");
-    pam_syslog(pamh, LOG_ERR, 
+    if (!configuration->quiet)
+      pam_syslog(pamh, LOG_ERR,
                "no valid certificate which meets all requirements found");
     goto auth_failed_nopw;
   }
@@ -509,7 +585,8 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     rv = get_private_key(ph);
     if (rv != 0) {
       ERR1("get_private_key() failed: %s", get_error());
-      pam_syslog(pamh, LOG_ERR, 
+      if (!configuration->quiet)
+        pam_syslog(pamh, LOG_ERR,
                  "get_private_key() failed: %s", get_error());
       goto auth_failed_nopw;
     }
@@ -519,17 +596,19 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     rv = get_random_value(random_value, sizeof(random_value));
     if (rv != 0) {
       ERR1("get_random_value() failed: %s", get_error());
-      pam_syslog(pamh, LOG_ERR, "get_random_value() failed: %s", get_error());
+      if (!configuration->quiet)
+        pam_syslog(pamh, LOG_ERR, "get_random_value() failed: %s", get_error());
       goto auth_failed_nopw;
     }
 
     /* sign random value */
     signature = NULL;
-    rv = sign_value(ph, chosen_cert, random_value, sizeof(random_value), 
+    rv = sign_value(ph, chosen_cert, random_value, sizeof(random_value),
 		    &signature, &signature_length);
     if (rv != 0) {
       ERR1("sign_value() failed: %s", get_error());
-      pam_syslog(pamh, LOG_ERR, "sign_value() failed: %s", get_error());
+      if (!configuration->quiet)
+        pam_syslog(pamh, LOG_ERR, "sign_value() failed: %s", get_error());
       goto auth_failed_nopw;
     }
 
@@ -544,7 +623,8 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
       close_pkcs11_session(ph);
       release_pkcs11_module(ph);
       ERR1("verify_signature() failed: %s", get_error());
-      pam_syslog(pamh, LOG_ERR, "verify_signature() failed: %s", get_error());
+      if (!configuration->quiet)
+        pam_syslog(pamh, LOG_ERR, "verify_signature() failed: %s", get_error());
       return PAM_AUTH_ERR;
     }
 
@@ -556,56 +636,61 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
    * fill in the environment variables.
    */
   snprintf(env_temp, sizeof(env_temp) - 1,
-	   "PKCS11_LOGIN_TOKEN_NAME=%.*s", 
-	   (sizeof(env_temp) - 1) - strlen("PKCS11_LOGIN_TOKEN_NAME="),
-	   get_slot_label(ph));
+	   "PKCS11_LOGIN_TOKEN_NAME=%.*s",
+	   (int)((sizeof(env_temp) - 1) - strlen("PKCS11_LOGIN_TOKEN_NAME=")),
+	   get_slot_tokenlabel(ph));
   rv = pam_putenv(pamh, env_temp);
 
   if (rv != PAM_SUCCESS) {
     ERR1("could not put token name in environment: %s",
-         pam_strerror(pamh, rv)); 
-    pam_syslog(pamh, LOG_ERR, "could not put token name in environment: %s",
-           pam_strerror(pamh, rv)); 
+         pam_strerror(pamh, rv));
+    if (!configuration->quiet)
+      pam_syslog(pamh, LOG_ERR, "could not put token name in environment: %s",
+           pam_strerror(pamh, rv));
   }
 
   issuer = cert_info((X509 *)get_X509_certificate(chosen_cert), CERT_ISSUER,
                      ALGORITHM_NULL);
   if (issuer) {
     snprintf(env_temp, sizeof(env_temp) - 1,
-	   "PKCS11_LOGIN_CERT_ISSUER=%.*s", 
-	   (sizeof(env_temp) - 1) - strlen("PKCS11_LOGIN_CERT_ISSUER="),
+	   "PKCS11_LOGIN_CERT_ISSUER=%.*s",
+	   (int)((sizeof(env_temp) - 1) - strlen("PKCS11_LOGIN_CERT_ISSUER=")),
 	   issuer[0]);
     rv = pam_putenv(pamh, env_temp);
   } else {
     ERR("couldn't get certificate issuer.");
-    pam_syslog(pamh, LOG_ERR, "couldn't get certificate issuer.");
+    if (!configuration->quiet)
+      pam_syslog(pamh, LOG_ERR, "couldn't get certificate issuer.");
   }
 
   if (rv != PAM_SUCCESS) {
     ERR1("could not put cert issuer in environment: %s",
-         pam_strerror(pamh, rv)); 
-    pam_syslog(pamh, LOG_ERR, "could not put cert issuer in environment: %s",
-           pam_strerror(pamh, rv)); 
+         pam_strerror(pamh, rv));
+    if (!configuration->quiet)
+      pam_syslog(pamh, LOG_ERR, "could not put cert issuer in environment: %s",
+           pam_strerror(pamh, rv));
   }
 
   serial = cert_info((X509 *)get_X509_certificate(chosen_cert), CERT_SERIAL,
                      ALGORITHM_NULL);
   if (serial) {
     snprintf(env_temp, sizeof(env_temp) - 1,
-	   "PKCS11_LOGIN_CERT_SERIAL=%.*s", 
-	   (sizeof(env_temp) - 1) - strlen("PKCS11_LOGIN_CERT_SERIAL="),
+	   "PKCS11_LOGIN_CERT_SERIAL=%.*s",
+	   (int)((sizeof(env_temp) - 1) - strlen("PKCS11_LOGIN_CERT_SERIAL=")),
 	   serial[0]);
     rv = pam_putenv(pamh, env_temp);
   } else {
     ERR("couldn't get certificate serial number.");
-    pam_syslog(pamh, LOG_ERR, "couldn't get certificate serial number.");
+    if (!configuration->quiet)
+      pam_syslog(pamh, LOG_ERR, "couldn't get certificate serial number.");
   }
 
   if (rv != PAM_SUCCESS) {
     ERR1("could not put cert serial in environment: %s",
-         pam_strerror(pamh, rv)); 
-    pam_syslog(pamh, LOG_ERR, "could not put cert serial in environment: %s",
-           pam_strerror(pamh, rv)); 
+         pam_strerror(pamh, rv));
+    if (!configuration->quiet)
+      pam_syslog(pamh, LOG_ERR, "could not put cert serial in environment: %s",
+           pam_strerror(pamh, rv));
   }
 
   /* close pkcs #11 session */
@@ -613,7 +698,8 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
   if (rv != 0) {
     release_pkcs11_module(ph);
     ERR1("close_pkcs11_session() failed: %s", get_error());
-    pam_syslog(pamh, LOG_ERR, "close_pkcs11_module() failed: %s", get_error());
+    if (!configuration->quiet)
+      pam_syslog(pamh, LOG_ERR, "close_pkcs11_module() failed: %s", get_error());
     return pkcs11_pam_fail;
   }
 
@@ -645,7 +731,7 @@ PAM_EXTERN int pam_sm_setcred(pam_handle_t *pamh, int flags, int argc, const cha
 PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t *pamh, int flags, int argc, const char **argv)
 {
   ERR("Warning: Function pm_sm_acct_mgmt() is not implemented in this module");
-  pam_syslog(pamh, LOG_WARNING, 
+  pam_syslog(pamh, LOG_WARNING,
              "Function pm_sm_acct_mgmt() is not implemented in this module");
   return PAM_SERVICE_ERR;
 }
@@ -653,7 +739,7 @@ PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t *pamh, int flags, int argc, const c
 PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags, int argc, const char **argv)
 {
   ERR("Warning: Function pam_sm_open_session() is not implemented in this module");
-  pam_syslog(pamh, LOG_WARNING, 
+  pam_syslog(pamh, LOG_WARNING,
              "Function pm_sm_open_session() is not implemented in this module");
   return PAM_SERVICE_ERR;
 }
@@ -661,7 +747,7 @@ PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags, int argc, cons
 PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh, int flags, int argc, const char **argv)
 {
   ERR("Warning: Function pam_sm_close_session() is not implemented in this module");
-  pam_syslog(pamh, LOG_WARNING, 
+  pam_syslog(pamh, LOG_WARNING,
            "Function pm_sm_close_session() is not implemented in this module");
   return PAM_SERVICE_ERR;
 }
@@ -671,7 +757,7 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc, const c
   char *login_token_name;
 
   ERR("Warning: Function pam_sm_chauthtok() is not implemented in this module");
-  pam_syslog(pamh, LOG_WARNING, 
+  pam_syslog(pamh, LOG_WARNING,
              "Function pam_sm_chauthtok() is not implemented in this module");
 
   login_token_name = getenv("PKCS11_LOGIN_TOKEN_NAME");
