@@ -17,7 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
 
-/* $Id: card_eventmgr.c 370 2009-09-02 12:32:55Z ludovic.rousseau $ */
+/* $Id: card_eventmgr.c 420 2010-04-10 14:38:23Z ludovic.rousseau $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -55,7 +55,7 @@ int timeout_count;
 int timeout_limit;
 int daemonize;
 int debug;
-char *cfgfile;
+const char *cfgfile;
 scconf_context *ctx = NULL;
 const scconf_block *root;
 SCARDCONTEXT hContext;
@@ -301,6 +301,7 @@ static void create_pidfile(char *filename)
 
 static void signal_trap(int sig)
 {
+    (void)sig;
     if (FALSE == AraKiri)
     {
 	DBG("Preparing to suicide");
